from importlib import resources
import pickle
import json
import dill
import pandas as pd
import numpy as np

# Class for OrganAge
class CreateOrganAgeObject:

    # init method or constructor
    def __init__(self,
                 path_scale_dict='v4_to_v4.1_scale_dict.json',
                 path_seed_dict='Bootstrap_and_permutation_500_seed_dict.json',
                 path_organ_plist_dict='tissue_pproteinlist_5k_dict_gtex_tissue_enriched_fc4_stable_proteins_seqid.json',
                 path_models_dir='data/ml_models/Covance/Zprot_stableps_perf95/'
                 ):

        self.data_and_model_paths = {"path_scale_dict": path_scale_dict,
                                     "path_seed_dict": path_seed_dict,
                                     "path_organ_plist_dict": path_organ_plist_dict,
                                     "path_models_dir": path_models_dir}
        self.load_data_and_models()

    def load_data_and_models(self):

        # Seqid:scale_factor dictionary
        scale_dict = json.load(resources.open_text("organage.data", self.data_and_model_paths["path_scale_dict"]))
        self.scale_dict = scale_dict

        # 500 bootstrap seed list
        all_seeds = json.load(resources.open_text("organage.data", self.data_and_model_paths["path_seed_dict"]))["BS_Seed"]
        self.all_seeds = all_seeds

        # organ:proteinlist dictionary
        organ_plist_dict = json.load(resources.open_text("organage.data", self.data_and_model_paths["path_organ_plist_dict"]))
        self.organ_plist_dict = organ_plist_dict

        # models
        models_dict = {}
        for organ in organ_plist_dict:
            models_dict[organ] = {}
            models_dict[organ]["aging_models"] = []

            # load protein scaler
            fn_protein_scaler = 'Covance_all_based_protein_zscore_scaler.pkl'
            loaded_model = pickle.loads(resources.read_binary('organage.data.ml_models.Covance.Zprot_stableps_perf95.' + organ, fn_protein_scaler))
            models_dict[organ]["prot_scaler"] = loaded_model

            # age gap scaler
            fn_agegap_scaler = 'Covance_all_Zprot_stableps_perf95_lasso_'+organ+'_agegap_zscore_scaler.pkl'
            loaded_model = pickle.loads(resources.read_binary('organage.data.ml_models.Covance.Zprot_stableps_perf95.' + organ, fn_agegap_scaler))
            models_dict[organ]["agegap_scaler"] = loaded_model

            # age prediction lowess
            fn_agepred_lowess = 'Covance_all_Zprot_stableps_perf95_lasso_' + organ + '_age_prediction_lowess.dill'
            loaded_model = dill.loads(resources.read_binary('organage.data.ml_models.Covance.Zprot_stableps_perf95.' + organ, fn_agepred_lowess))
            models_dict[organ]["age_prediction_lowess"] = loaded_model

            # load all aging models
            for seed in all_seeds:
                fn_aging_model = 'Covance_all_Zprot_stableps_perf95_lasso_'+organ+'_seed'+str(seed)+'_aging_model.pkl'
                loaded_model = pickle.loads(resources.read_binary('organage.data.ml_models.Covance.Zprot_stableps_perf95.'+ organ, fn_aging_model))
                models_dict[organ]["aging_models"].append(loaded_model)

        # save to object
        self.models_dict = models_dict


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['organage',
 'organage.data',
 'organage.data.ml_models',
 'organage.data.ml_models.Covance',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Adipose',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Artery',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Brain',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Heart',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Immune',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Intestine',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Kidney',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Liver',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Lung',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Muscle',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Organismal',
 'organage.data.ml_models.Covance.Zprot_stableps_perf95.Pancreas',
 'organage.data.ml_models.KADRC']

package_data = \
{'': ['*'],
 'organage.data.ml_models.KADRC': ['Zprot_stableps_perf95/Adipose/*',
                                   'Zprot_stableps_perf95/Artery/*',
                                   'Zprot_stableps_perf95/Brain/*',
                                   'Zprot_stableps_perf95/Heart/*',
                                   'Zprot_stableps_perf95/Immune/*',
                                   'Zprot_stableps_perf95/Intestine/*',
                                   'Zprot_stableps_perf95/Kidney/*',
                                   'Zprot_stableps_perf95/Liver/*',
                                   'Zprot_stableps_perf95/Lung/*',
                                   'Zprot_stableps_perf95/Muscle/*',
                                   'Zprot_stableps_perf95/Organismal/*',
                                   'Zprot_stableps_perf95/Pancreas/*',
                                   'Zprot_stableps_perf95_fiba/DementiaAdipose/*',
                                   'Zprot_stableps_perf95_fiba/DementiaArtery/*',
                                   'Zprot_stableps_perf95_fiba/DementiaBrain/*',
                                   'Zprot_stableps_perf95_fiba/DementiaHeart/*',
                                   'Zprot_stableps_perf95_fiba/DementiaImmune/*',
                                   'Zprot_stableps_perf95_fiba/DementiaIntestine/*',
                                   'Zprot_stableps_perf95_fiba/DementiaKidney/*',
                                   'Zprot_stableps_perf95_fiba/DementiaLiver/*',
                                   'Zprot_stableps_perf95_fiba/DementiaLung/*',
                                   'Zprot_stableps_perf95_fiba/DementiaMuscle/*',
                                   'Zprot_stableps_perf95_fiba/DementiaPancreas/*']}

install_requires = \
['dill>=0.3.6', 'pandas>=1.5.3', 'scikit-learn==1.0.2']

setup_kwargs = {
    'name': 'organage',
    'version': '0.1.1',
    'description': 'A package to pestimate organ-specific biological age using SomaScan plasma proteomics data',
    'long_description': '# organage\n\nA package to pestimate organ-specific biological age using SomaScan plasma proteomics data\n\n## Installation\n\n```bash\n$ pip install organage\n```\n\n## Usage\n\n- TODO\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`organage` was created by Hamilton Oh. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`organage` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Hamilton Oh',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9',
}


setup(**setup_kwargs)
